from typing import Dict, Optional

from arcane.core import UserRightsEnum, RightsLevelEnum, BadRequestError, BaseAccount
from arcane.requests import call_get_route


def get_google_analytics_v4_account(
    base_account: BaseAccount,
    clients_service_url: Optional[str] = None,
    firebase_api_key: Optional[str] = None,
    gcp_service_account: Optional[str] = None,
    auth_enabled: bool = True
) -> Dict:
    """Call get endpoint to retrieve ga4 account

    Args:
        base_account (BaseAccount): base account to get
        clients_service_url (Optional[str], optional): clients service url to call. Defaults to None.
        firebase_api_key (Optional[str], optional): needed for calling api. Defaults to None.
        gcp_service_account (Optional[str], optional): needed for calling api. Defaults to None.
        auth_enabled (bool, optional): Boolean to know if we should use token while calling api. Defaults to True.

    Raises:
        BadRequestError: The request does not comply function requirement. See error message for more info

    Returns:
        Dict: ga4 account
    """
    if not (clients_service_url and firebase_api_key and gcp_service_account):
        raise BadRequestError('clients_service_url or firebase_api_key or gcp_service_account should not be None if google analytics v4 account is not provided')
    url = f"{clients_service_url}/api/google-analytics-account/v4?account_id={base_account['id']}&client_id={base_account['client_id']}"
    accounts = call_get_route(
        url,
        firebase_api_key,
        claims={'features_rights':{UserRightsEnum.AMS_GTP: RightsLevelEnum.VIEWER}, 'authorized_clients': ['all']},
        auth_enabled=auth_enabled,
        credentials_path=gcp_service_account
    )
    if len(accounts) == 0:
        raise BadRequestError(f'Error while getting google analytics v4 account with: {base_account}. No account corresponding.')
    elif len(accounts) > 1:
        raise BadRequestError(f'Error while getting google analytics v4 account with: {base_account}. Several account corresponding: {accounts}')

    return accounts[0]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['arcane', 'arcane.ga4']

package_data = \
{'': ['*']}

install_requires = \
['arcane-core==1.7.0',
 'arcane-credentials==0.1.0',
 'arcane-datastore==1.1.0',
 'arcane-requests==0.3.0',
 'google-analytics-admin==0.8.1',
 'google-api-core==1.31.5',
 'google-auth>=1.30.0,<2.0.0']

setup_kwargs = {
    'name': 'arcane-ga4',
    'version': '0.1.1',
    'description': 'Utility functions for ga4 api call',
    'long_description': '# Arcane ga4 README\n\nThis package helps us to interact with Google Analytics V4 API:\n\n    + [Analytics Data API](https://developers.google.com/analytics/devguides/reporting/data/v1)\n    + [Google Analytics Admin API](https://developers.google.com/analytics/devguides/config/admin/v1)\n\n\n## Release history\nTo see changes, please see CHANGELOG.md\n',
    'author': 'Arcane',
    'author_email': 'product@arcane.run',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Optional, cast, Dict

from google.oauth2 import service_account
from google.analytics.admin import AnalyticsAdminServiceClient
from google.api_core.exceptions import PermissionDenied, InvalidArgument

from arcane.ga4.exception import GoogleAnalyticsV4AccountException
from arcane.core import BadRequestError, BaseAccount
from arcane.credentials import get_user_decrypted_credentials
from arcane.datastore import Client as DatastoreClient

from .helpers import get_google_analytics_v4_account


class GaV4Client:
    def __init__(
        self,
        gcp_service_account: str,
        property_id: str,
        base_account: Optional[BaseAccount] = None,
        ga_v4_account: Optional[Dict] = None,
        datastore_client: Optional[DatastoreClient] = None,
        gcp_project: Optional[str] = None,
        secret_key_file: Optional[str] = None,
        firebase_api_key: Optional[str] = None,
        auth_enabled: bool = True,
        clients_service_url: Optional[str] = None,
        user_email: Optional[str] = None
    ):

        self.property_id = property_id
        scopes = ['https://www.googleapis.com/auth/analytics.readonly']
        if gcp_service_account and (ga_v4_account or base_account or user_email):
            if user_email:
                creator_email = user_email
            else:
                if ga_v4_account is None:
                    base_account = cast(BaseAccount, base_account)
                    ga_v4_account = get_google_analytics_v4_account(
                        base_account=base_account,
                        clients_service_url=clients_service_url,
                        firebase_api_key=firebase_api_key,
                        gcp_service_account=gcp_service_account,
                        auth_enabled=auth_enabled
                    )

                creator_email = ga_v4_account['creator_email']

            if creator_email is not None:
                if not secret_key_file:
                    raise BadRequestError('secret_key_file should not be None while using user access protocol')

                self.credentials = get_user_decrypted_credentials(
                    user_email=creator_email,
                    secret_key_file=secret_key_file,
                    gcp_credentials_path=gcp_service_account,
                    gcp_project=gcp_project,
                    datastore_client=datastore_client
                )
            else:
                self.credentials = service_account.Credentials.from_service_account_file(gcp_service_account, scopes=scopes)
        elif gcp_service_account:
            ## Used when posting an account using our credential (it is not yet in our database)
            self.credentials = service_account.Credentials.from_service_account_file(gcp_service_account, scopes=scopes)
        else:
            raise BadRequestError('one of the following arguments must be specified: gcp_service_account and (google_ads_account or base_account or user_email)')



    def get_property_name(self) -> str:
        client = AnalyticsAdminServiceClient(credentials=self.credentials)

        try:
            property_ = client.get_property(name=f"properties/{self.property_id}")
        except PermissionDenied:
            raise GoogleAnalyticsV4AccountException(f'We cannot access your property with the id: {self.property_id} from the Arcane account. Are you sure you granted access and gave the correct ID?')
        except InvalidArgument as err:
            raise GoogleAnalyticsV4AccountException(str(err))
        return str(property_.display_name)
